// -------------------------------------------------------------------
//  Copyright (c) 2011-2013 TIBCO Software, Inc.
//  All rights reserved.
//  For more information, please contact:
//  TIBCO Software Inc., Palo Alto, California, USA
//	Oracle sample code by Stefano Ghio - sghio@tibco.com
// -------------------------------------------------------------------
package com.tibco.shared_all.persistence;

import com.tibco.as.space.*;
import com.tibco.as.space.persistence.*;

import java.sql.SQLException;
import java.util.Collection;
import java.util.Properties;

public class ASPersister implements Persister
{
    Metaspace ms;

    public ASPersister (Metaspace ms)
    {
        this.ms = ms;
    }

    public static int LOAD_BATCH_SIZE = 100;

    // default database name, database host name, database user name, database user password
    public static String DB_NAME = "orcl";
    public static String DB_HOST = "localhost";
    public static String DB_PORT = "1521";
    public static String USER_NAME = "admin";
    public static String USER_PASS = "12345";
    private ASJdbcConnection db;

    public static String JDBC_DRIVER = "oracle.jdbc.OracleDriver";
    public static String JDBC_URL = "jdbc:oracle:thin:@" + DB_HOST + ":"+DB_PORT+":" + DB_NAME;

    public ActionResult onOpen (OpenAction openAction)
    {
        System.out.println("onOpen for " + openAction.getSpaceName());
        try
        {
            String name = openAction.getSpaceName();
            SpaceDef spaceDef = ms.getSpaceDef(name);

            String driverCls = JDBC_DRIVER;
            String connectionURL = JDBC_URL;

            Properties properties = new Properties();
            properties.put("user", USER_NAME);
            properties.put("password", USER_PASS);

            db = new ASJdbcConnection(driverCls, connectionURL, properties);

            // you can overwrite any SQL mapping differences here instead of
            // writing your own database connection class
            // Ex:
            //    db.setSQLMapping(FieldType.BLOB, "BLOB");

            db.init(spaceDef);
            return ActionResult.create();
        }
        catch (ASException as)
        {
            System.out.println("ASException: " + as);
        }
        catch (ClassNotFoundException ex)
        {
            System.out.println("Driver class not found " + ex);
        }
        catch (SQLException ex)
        {
            System.out.println("Sql exception" + ex);
        }

        return ActionResult.create().setFailed();
    }

    public ActionResult onLoad (LoadAction loadAction)
    {
        System.out.println("onLoad for " + loadAction.getSpaceName());
        Collection<Tuple> tuples = null;
        int batchSize = LOAD_BATCH_SIZE;
        int offset = 0;

        String name = loadAction.getSpaceName();
        try
        {
            Space space = ms.getSpace(name);
            do
            {
                tuples = db.load(batchSize, offset);
                if (tuples.size() > 0)
                {
                    System.out.println("Loading " + tuples.size() + " tuples");
                    SpaceResultList list = space.loadAll(tuples);
                    if (list.hasError())
                    {
                        System.out.println("LoadAll completed with errors");
                    }
                }
                if (tuples.size() < batchSize)
                {
                    break;
                }

                offset += batchSize;
            }
            while (!tuples.isEmpty());
            space.close();
        }
        catch (ASException as)
        {
            as.printStackTrace();
        }
        return ActionResult.create();
    }

    public ActionResult onClose (CloseAction closeAction)
    {
        System.out.println("onClose");
        db.close();
        return ActionResult.create();
    }

    public ActionResult onWrite (WriteAction writeAction)
    {
        System.out.println("onWrite for " + writeAction.getSpaceName());
        for (WriteOp operation : writeAction.getOps())
        {
            switch (operation.getType())
            {
                case PUT:
                    db.process((PutOp) operation);
                    break;
                case TAKE:
                    db.process((TakeOp) operation);
                    break;
            }
        }
        return ActionResult.create();
    }

    @Override
    public ActionResult onAlter (AlterAction alterAction)
    {
        System.out.println("SpaceDef updated");
        System.out.println(alterAction.getSpaceDef());

        return null;
    }

    public ActionResult onRead (ReadAction readAction)
    {
        System.out.println("OnRead for " + readAction.getSpaceName());
        Tuple tuple = db.loadTuple(readAction.getTuple());
        return ActionResult.create(tuple);
    }


}
