// -------------------------------------------------------------------

//  Copyright (c) 2011-2013 TIBCO Software, Inc.
//  All rights reserved.
//  For more information, please contact:
//  TIBCO Software Inc., Palo Alto, California, USA
//	Oracle sample code by Stefano Ghio - sghio@tibco.com
// -------------------------------------------------------------------

/*
 * This example shows how to create a persisted space using the external shared-all persistence feature
 *
 * The example defines a space, joins it as a leech and registers itself as a persister for the space
 *
 *
 */
package com.tibco.shared_all.test;

import com.tibco.shared_all.base.ASExampleBase;
import com.tibco.as.space.*;
import com.tibco.as.space.Member.DistributionRole;
import com.tibco.as.space.persistence.Persister;

import com.tibco.shared_all.persistence.ASPersister;

import java.io.BufferedReader;
import java.io.InputStreamReader;

public class ASPersistence extends ASExampleBase
{

    public static void main (String[] args)
    {
        ASPersistence example = null;
        try
        {
            example = new ASPersistence(args);
            example.init();
            example.run();
        }
        catch (ASException as)
        {
            as.printStackTrace();
        }
        finally
        {
            if (example != null)
                example.cleanup();
        }
    }

    Metaspace metaspace;
    Space space;
    Persister persister;

    public ASPersistence (String[] args)
    {
        super(args);
        ASCommon.setLogLevel(LogLevel.INFO);
    }

    public void init() throws ASException
    {
        // parse the command line arguments
        this.parseArgs();
        // connect to metaspace
        metaspace = connectMetaspace();
        // define space name to be persisted
        setDefaultSpaceName("shared_all_persisted");
        // get space definition
        SpaceDef spaceDef = getSpaceDef();
        spaceDef.setPersistenceType(SpaceDef.PersistenceType.SHARE_ALL);
        // define space
        metaspace.defineSpace(spaceDef);
        // get space
        updateRole(DistributionRole.LEECH);
        space = getSpace(metaspace);
        //Enable JDBC persister
        persister = space.setPersister(new ASPersister(metaspace));
    }

    public void run ()
    {
        try
        {
            BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
            System.out.println("Type 'quit' to quit");
            while (true)
            {
                String input = br.readLine();
                if (input.equals("quit"))
                {
                    break;
                }
            }
        }
        catch (Exception ex)
        {
            // do nothing, just exit
        }
    }

    public void cleanup()
    {
        try
        {
            if (space != null && persister != null) space.stopPersister(persister);
            if (space != null) space.close();
            if (metaspace != null) metaspace.closeAll();
        }
        catch (ASException ex)
        {
            ex.printStackTrace();
        }
    }
}
